package com.pointrlabs.sample.map;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.MenuItem;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.pointrlabs.core.geojson.Feature;
import com.pointrlabs.core.management.PermissionManager;
import com.pointrlabs.core.management.Pointr;
import com.pointrlabs.core.management.PositionManager;
import com.pointrlabs.core.management.models.Building;
import com.pointrlabs.core.management.models.Level;
import com.pointrlabs.core.management.models.Site;
import com.pointrlabs.core.map.models.MapTrackingMode;
import com.pointrlabs.core.map.models.PTRMapViewBounds;
import com.pointrlabs.core.map.models.PTRMapViewMarker;
import com.pointrlabs.core.map.models.events_listeners.MapEventsListener;
import com.pointrlabs.core.map.viewmodels.PTRMapWidgetConfiguration;
import com.pointrlabs.core.map.views.PTRMapFragment;
import com.pointrlabs.core.map.views.PTRMapWidgetFragment;
import com.pointrlabs.core.poi.models.Poi;
import com.pointrlabs.core.positioning.model.CalculatedLocation;
import com.pointrlabs.core.positioning.model.PositioningTypes;
import com.pointrlabs.sample.R;

import java.util.List;
import java.util.Objects;

public class MapActivity extends AppCompatActivity implements MapEventsListener {

    private static final String BUNDLE_BUILDING_ID = "BUNDLE_BUILDING_ID";

    private Building building;

    /**
     * Creates an intent to start this activity
     *
     * @param venueId to load and display the {@link PTRMapWidgetFragment}
     * @return intent to start {@link }
     */
    public static Intent newIntent(@NonNull Context context,
                                   int venueId) {
        final Intent intent = new Intent(context, MapActivity.class);
        intent.putExtra(BUNDLE_BUILDING_ID, venueId);
        return intent;
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_map_widget);

        Objects.requireNonNull(getSupportActionBar()).setDisplayHomeAsUpEnabled(true);
        int buildingInternalIdentifier = getIntent().getIntExtra(BUNDLE_BUILDING_ID, 0);
        // if you want to delegate activity recreation handling to PointrSDK
        // get the widget instance after calling show method
        final PTRMapWidgetConfiguration configuration =
                PTRMapWidgetConfiguration.defaultConfiguration();
        final PTRMapWidgetFragment widgetFragment = PTRMapWidgetFragment
                .newInstance(configuration)
                .show(getSupportFragmentManager(), R.id.fragment_container);
        assert Pointr.getPointr() != null;
        building = Objects.requireNonNull(Objects.requireNonNull(Pointr.getPointr().getSiteManager()).getBuilding(buildingInternalIdentifier));
        widgetFragment.loadAndSet(building);
        widgetFragment.addListener(this);
        this.setTitle(building.getTitle());
        PermissionManager permissionManager = Pointr.getPointr().getPermissionManager();
        if(!permissionManager.getHasLocationPermissionWhileInUse()) {
            permissionManager.requestLocationPermissionForWhileInUse(this, 1);
        } else if(!permissionManager.getHasLocationPermissionAlways()) {
            permissionManager.requestLocationPermissionForAlways(this, 2);
        } else if (!permissionManager.getHasBluetoothPermission()) {
            permissionManager.requestBluetoothPermission(this, 3);
        } else if (!permissionManager.isLocationServicesEnabled()) {
            permissionManager.requestLocationService(this);
        } else if (!permissionManager.isBluetoothServicesEnabled() && permissionManager.getHasBluetoothPermission()) {
            permissionManager.requestBluetoothService();
        }

    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        PermissionManager permissionManager = Pointr.getPointr().getPermissionManager();
        switch (requestCode) {
            case 1:
            case 421903:
                if(!permissionManager.getHasLocationPermissionAlways()) {
                    permissionManager.requestLocationPermissionForAlways(this, 2);
                } else if (!permissionManager.getHasBluetoothPermission()) {
                    permissionManager.requestBluetoothPermission(this, 3);
                } else if (!permissionManager.isLocationServicesEnabled()) {
                    permissionManager.requestLocationService(this);
                } else if (!permissionManager.isBluetoothServicesEnabled() && permissionManager.getHasBluetoothPermission()) {
                    permissionManager.requestBluetoothService();
                }
                break;
            case 2:
                if (!permissionManager.getHasBluetoothPermission()) {
                    permissionManager.requestBluetoothPermission(this, 3);
                } else if (!permissionManager.isLocationServicesEnabled()) {
                    permissionManager.requestLocationService(this);
                } else if (!permissionManager.isBluetoothServicesEnabled() && permissionManager.getHasBluetoothPermission()) {
                    permissionManager.requestBluetoothService();
                }
                break;
            case 3:
            case 421904:
                if (!permissionManager.isLocationServicesEnabled()) {
                    permissionManager.requestLocationService(this);
                } else if (!permissionManager.isBluetoothServicesEnabled() && permissionManager.getHasBluetoothPermission()) {
                    permissionManager.requestBluetoothService();
                }
                break;
            case 42191:
                if (!permissionManager.isBluetoothServicesEnabled() && permissionManager.getHasBluetoothPermission()) {
                    permissionManager.requestBluetoothService();
                }
                break;
        }

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        PermissionManager permissionManager = Pointr.getPointr().getPermissionManager();
        switch(requestCode) {
            case 42191:
                if (!permissionManager.isBluetoothServicesEnabled() && permissionManager.getHasBluetoothPermission()) {
                    permissionManager.requestBluetoothService();
                }
                break;
        }
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            onBackPressed();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void mapDidStartLoading(@NonNull PTRMapFragment ptrMapFragment) {

    }

    @Override
    public void mapDidEndLoading(@NonNull PTRMapFragment ptrMapFragment) {

    }

    @Override
    public void mapDidFailToLoad(@NonNull PTRMapFragment ptrMapFragment, @NonNull Exception e) {

    }

    @Override
    public void mapDidReceiveTapOnPoi(@NonNull PTRMapFragment ptrMapFragment, @NonNull Poi poi) {

    }

    @Override
    public void mapDidReceiveTap(@NonNull PTRMapFragment ptrMapFragment, @NonNull CalculatedLocation calculatedLocation) {

    }

    @Override
    public void mapDidReceiveTapOnMarker(@NonNull PTRMapViewMarker ptrMapViewMarker) {

    }

    @Override
    public void mapDidChangeLevel(@NonNull PTRMapFragment ptrMapFragment, @NonNull Level level, boolean b) {

    }

    @Override
    public void mapDidUpdateStyle(@NonNull PTRMapFragment ptrMapFragment) {

    }

    @Override
    public void mapDidChangeTrackingMode(@NonNull PTRMapFragment ptrMapFragment, @NonNull MapTrackingMode mapTrackingMode, @NonNull MapTrackingMode mapTrackingMode1) {

    }

    @Override
    public void mapDidDetectFocusedSite(@NonNull PTRMapFragment ptrMapFragment, @Nullable Site site, @NonNull List<? extends Building> buildings) {
        if(buildings.size() == 0) {
            setTitle("");
            return;
        }
        if(buildings.contains(building)) return;
        building = buildings.get(0);
        setTitle(building.getTitle());
    }

    @Override
    public void mapDidDetectVisibleSites(@NonNull PTRMapFragment ptrMapFragment, @NonNull List<? extends Site> list, @NonNull List<? extends Building> list1) {

    }

    @Override
    public void mapDidScrollTo(@NonNull PTRMapFragment ptrMapFragment, @NonNull PTRMapViewBounds ptrMapViewBounds) {

    }

    @Override
    public void mapDidRotate(@NonNull PTRMapFragment ptrMapFragment, double v) {

    }

    @Override
    public void mapDidZoom(@NonNull PTRMapFragment ptrMapFragment, double v) {

    }

    @Override
    public void mapDidReceiveFling(@NonNull PTRMapFragment ptrMapFragment) {

    }

    @Override
    public void mapDidChangeRegion(@NonNull PTRMapFragment ptrMapFragment, boolean b) {

    }

    @Override
    public void mapDidReceiveTapOnFeature(@NonNull PTRMapFragment ptrMapFragment, @NonNull Feature feature) {

    }
}
